package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesLoadingVehiclesOrderService;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesLoadingVehiclesOrder;
import cn.estsh.i3plus.pojo.mes.bean.MesLoadingVehiclesOrderDetail;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class MesLoadingVehiclesOrderService extends BaseMesService<MesLoadingVehiclesOrder> implements IMesLoadingVehiclesOrderService {


    @Autowired
    private MesLoadingVehiclesOrderDetailService detailService;

    @Override
    public MesLoadingVehiclesOrder insert(MesLoadingVehiclesOrder bean) {
        //插入前构造并校验数据
        onInsertBean(bean);

        List<MesLoadingVehiclesOrderDetail> detailList = new ArrayList<>();
        MesLoadingVehiclesOrder finalBean = bean;
        bean.getShippingOrderNoList().forEach(k -> {
            MesLoadingVehiclesOrderDetail detail = new MesLoadingVehiclesOrderDetail();

            BeanUtils.copyProperties(k, detail, "id");
            detail.setShippingOrderNo(finalBean.getLoadingOrderNo());
            detail.setWeatherCondition(finalBean.getWeatherCondition());
            detailList.add(detail);
        });
        String customerCode = bean.getShippingOrderNoList().get(0).getCustomerCode();
        String customerName = bean.getShippingOrderNoList().get(0).getCustomerName();
        String customerOrganizeCode = bean.getShippingOrderNoList().get(0).getCustomerOrganizeCode();
        String customerOrganizeName = bean.getShippingOrderNoList().get(0).getCustomerOrganizeName();
        bean.setCustomerCode(customerCode);
        bean.setCustomerName(customerName);
        bean.setCustomerOrganizeCode(customerOrganizeCode);
        bean.setCustomerOrganizeName(customerOrganizeName);
        //TODO 状态待确定
//        bean.setStatus(MesExtEnumUtil.);
        detailService.insertBatch(detailList);
        bean = baseRDao.insert(bean);
        return bean;
    }

    @Override
    public void deleteWeaklyByIds(Long[] ids, String userName) {

        for (Long id : ids) {
            MesLoadingVehiclesOrder bean = baseRDao.getById(id);
            if (StringUtils.isEmpty(bean)) continue;
            DdlPackBean detailPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getLoadingOrderNo(), "loadingOrderNo", detailPackBean);
            List<MesLoadingVehiclesOrderDetail> detailDeleteList = detailService.findAll(detailPackBean);
            detailDeleteList.forEach(k -> {
                k.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
            });
            bean.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            ConvertBean.serviceModelUpdate(bean, AuthUtil.getSessionUser().getUserName());
            baseRDao.update(bean);
            detailService.updates(detailDeleteList);
        }
    }

    protected void onInsertBean(MesLoadingVehiclesOrder item) {

        List<String> shippingOrderNoList = new ArrayList<>();
        // 数据校验
        ValidatorBean.checkNotNull(item.getLoadingOrderNo(), "装车单号不能为空");
        ValidatorBean.checkNotNull(item.getWeatherCondition(), "天气不能为空");
        ValidatorBean.checkNotNull(item.getShippingOrderNoList(), "发运单不能为空");
        item.getShippingOrderNoList().forEach(k -> {
            if (k.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue() || k.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()) {
                shippingOrderNoList.add(k.getShippingOrderNo());
            }
        });

        if (!CollectionUtils.isEmpty(shippingOrderNoList)) {
            throw ImppExceptionBuilder.newInstance()
                    .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                    .setErrorDetail(String.format("发运单【%s】状态不是【%s】或【%s】,请重新选择数据", shippingOrderNoList, MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue(),
                            MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()))
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .build();
        }

    }

    protected void setPackQueryBean(MesLoadingVehiclesOrder bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getLoadingOrderNo(), "loadingOrderNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustomerCode(), "customerCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustomerOrganizeCode(), "customerOrganizeCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCarNo(), "carNo", packBean);
    }
}
